import BaseObject
import Attribute

class Container(BaseObject.BaseObject):
  def __init__(self, *args, **kw):
    BaseObject.BaseObject.__init__(self)
    self._members = []
    self._internalLabel = ""

  def arguments_(self, arguments):
    if arguments:
      for argument in arguments:
        self.append(argument)

  # --- label -----------------------------------

  def internalLabel(self): return self._internalLabel
  def internalLabel_(self, o): self._internalLabel = o; return self

  # --- members -----------------------------------

  def members(self): return self._members
  def members_(self, o): self._members = o; return self

  def membersHtml(self):
    s = ""
    for member in self.members():
      if member is self:
        raise MemberEqualSelf
      else:
        if hasattr(member, 'htmlContent'):
          content = member.htmlContent()
          s = s + content
        else:
          s = s + str(member)
    return s

  def append(self, o): # for compatability with HTMLgen 
    self.members().append(o); 
    return o 

  def prepend(self, o): 
    self.members().insert(0, o)
    return o 

  def append_(self, o): 
    self.members().append(o); 
    return o

  def prepend_(self, o): 
    self.members().prepend(o)
    return o 

  def content_(self, o): 
    self.members_([o]); 
    return self

  def removeMember_(self, o):
    self.members().remove(o)
    return o

  def addMembers_(self, list):
    self.members_(self.members() + list)

  def __call__(self, text):
    clone = self.clone()
    clone.members_([text])
    return clone

  def htmlContentWithMembers_(self, tempMembers):
    members = self.members()
    self.members_(tempMembers)
    aString = self.htmlContent()
    self.members_(members)
    return aString

  # --- finding members -------------------------------------

  def membersWith_equalTo_(self, methodName, value):
    results = []
    for member in self.members():
      if type(member) == type(self):
        if member.respondsTo_(methodName):
          if member.perform_(methodName) == value:
            results.append(member)
    return results

  def recursiveMembersWith_equalTo_(self, methodName, value):
    results = []
    for member in self.members():
      if type(member) == type(self):
        if member.respondsTo_("allMembersWith_equalTo_"):
          results = results + member.recursiveMembersWith_equalTo_(methodName, value)
    return results

  # --- finding members utilities ----------------------------

  def membersWithTag_(self, tagName):
    return self.membersWith_equalTo_("tagName", tagName)

  # --- rendering -------------------------------------

  def htmlContent(self):
    return self.membersHtml()   

  def __str__(self):
    return self.htmlContent()

  def writeTo_(self, file):
    file.write(str(self))
    return self
