import BaseObject
import Attribute
import Container
import string

class Group(Container.Container):
  def __init__(self, *args, **kw):
    Container.Container.__init__(self)
    self._tagName = ""
    self._attributes = [] # list of Attribute objects
    self._modifiers = []
    self.keywords_(kw)

  def keywords_(self, kw):
    for name, value in kw.items():
      self.set_value_(name, value)

  # --- clone -----------------------------------
 
  def clone(self):
    clone = self.shallowCopy()
    clone.members_([])
    import copy
    clone.attributes_(copy.copy(clone.attributes()))
    return clone

  # --- tag -----------------------------------

  def tagName(self): return self._tagName
  def tagName_(self, o): self._tagName = o; return self

  def beginTag(self):
    return "<%s%s>" % (self.tagName(), self.attributesHtml())
    
  def endTag(self):
    return "</%s>" % self.tagName()

  # --- attributes -----------------------------------

  def attributes(self): return self._attributes
  def attributes_(self, o): self._attributes = o; return self

  def set_value_(self, name, value):
    return self.setAttributeName_value_(name, value)

  def setAttributeName_value_(self, name, value):
    attribute = Attribute.Attribute(name, value)
    return self.addAttribute_(attribute)

  def addAttribute_(self, attribute):
    old = self.attributeNamed_(attribute.name())
    self.removeAttribute_(old)
    self.attributes().append(attribute)
    return attribute

  def removeAttribute_(self, attribute):
    if attribute: self.attributes().remove(attribute)
    return self

  def attributeNamed_(self, name):
    for attribute in self.attributes():
      if attribute.name() == name: return attribute
    return None

  def attributesHtml(self):
    s = ""
    for attribute in self.attributes():
      if attribute.htmlContent():
        s = s + " " + attribute.htmlContent()
    return s

  # --- rendering -------------------------------------

  def modifiedMembersHtml(self):
    s = "%s%s%s" % (self.modifierBeginTags(), self.membersHtml(), self.modifierEndTags())
    return s

  def htmlContent(self):
    if self.respondsTo_("prepareToRender"):
      self.members_([])
      self.prepareToRender()
    s = self.modifiedMembersHtml()
    if self.tabString():
      tabString = self.tabString()
      s = tabString + string.replace(s, '\n', '\n' + tabString)
      s = "\n%s\n%s\n%s" % (self.beginTag(), s, self.endTag())
      if self.endTag(): s = s + '\n'
      s = string.replace(s, '\n' + tabString + '\n', tabString + '\n')
    else:
      s = "%s%s%s" % (self.beginTag(), s, self.endTag())
    #import sys
    #sys.stderr.write(">>>  " + s + "\n")
    return s

  def tabString(self): return '' 
  #def tabString(self): return '   '

  def __str__(self):
    return self.htmlContent()

  # --- modifiers -------------------------------------

  def modifiers_(self, o): self._modifiers = o; return self
  def modifiers(self): return self._modifiers

  def addModifier_(self, o): self.modifiers().append(o)

  def modifierBeginTags(self):
    s = ""
    for modifier in self.modifiers():
      s = s + modifier.beginTag()
    return s

  def modifierEndTags(self):
    s = ""
    if not self.modifiers(): return ""
    import copy
    mods = copy.copy(self.modifiers())
    mods.reverse()
    for modifier in mods:
      s = s + modifier.endTag()
    return s

