
import FormElement
import Option

class Select(FormElement.FormElement):
  def __init__(self, *args, **kw):
    FormElement.FormElement.__init__(self)
    self.tagName_("select")
    self.setAttributeName_value_("multiple", None).showsValue_(0)
    self.setAttributeName_value_("name", "")
    self.setAttributeName_value_("size", None)
    self.keywords_(kw)
    self.arguments_(args)
    self._selectedNames = []
    if len(args) > 0: self.optionNames_(args[0])
    if len(args) > 1: self.selectedNames_(args[1])

  def keywords_(self, kw):
    FormElement.FormElement.keywords_(self, kw)
    if kw.get("selected", None):
      self.selectedNames_(kw["selected"])

  def name_(self, o): self.set_value_("name", o); return self
  def name(self): return self.attributeNamed_("name").value()

  def size_(self, o): self.set_value_("size", o); return self
  def size(self): return self.attributeNamed_("size").value()

  def multiple_(self, o): self.set_value_("multiple", o); return self
  def multiple(self): return self.attributeNamed_("multiple").value()

  def options_(self, o): return self.members_(o)
  def options(self): return self.members()

  def selectedNames_(self, o): self._selectedNames = o; return self
  def selectedNames(self): return self._selectedNames

  def newOption(self, *args, **kw):
    option = Option.Option()
    option.arguments_(args)
    option.keywords_(kw)
    self.append(option)
    return option

  def optionsDict_(self, dict):
    self.optionsNameKeyDict_(dict)
    return self

  def optionsNameKeyDict_(self, dict):
    self.options_([])
    keys = dict.keys()
    keys.sort()
    for key in keys:
      option = self.newOption()
      option.name_(key)
      option.value_(dict[key])

  def optionsKeyNameDict_(self, dict):
    self.options_([])
    keys = dict.keys()
    keys.sort()
    for key in keys:
      option = self.newOption()
      option.name_(dict[key])
      option.value_(key)


  # --- names ----------------------------------------

  def optionNames_(self, names):
    self.options_([])
    for name in names:
      option = self.newOption().name_(name)

  def selectOptionsWithNames_(self, names):
    for option in self.options():
      option.selected_((option.name() in names))

  def selectOptionWithName_(self, name):
    for option in self.options():
      option.selected_((option.name() == name))

  def optionNames(self):
    names = []
    for option in self.options():
      names.append(option.name())
    return names

  # --- value ----------------------------------------

  def selectOptionWithValue_(self, value):
    for option in self.options():
      if (option.value() == value):
        option.selected_(1)
      else:
        option.selected_(0)

  def selectOptionsWithValues_(self, values):
    for option in self.options():
      if (option.value() in values):
        option.selected_(1)
      else:
        option.selected_(0)

  # --- selected --------------------------------------

  def selectedOptions(self):
    selected = []
    for option in self.options():
      if option.selected(): selected.append(option)
    return selected

  def selectedOptionNames(self):
    selected = []
    for option in self.options():
      if option.selected(): selected.append(option.name())
    return selected

  def unselectedOptions(self):
    selected = []
    for option in self.options():
      if not option.selected(): selected.append(option)
    return selected

  def moveSelectedOptionsToTop(self):
    self.options_(self.selectedOptions() + self.unselectedOptions())
    return self

  # --- rendering --------------------------------

  def htmlContent(self):
    self.attributeNamed_("multiple").showsValue_(0) # hack
    if self.multiple():
      self.moveSelectedOptionsToTop()

    if self.selectedNames():
      self.selectOptionsWithNames_(self.selectedNames())

    if not self.isEditable():
      s = ""
      for option in self.options():
        if option.selected():
          s = s + option.name() + "<BR>"      
      return s
    return FormElement.FormElement.htmlContent(self)
